@@ -14,6 +14,7 @@ from six import string_types,text_type, next
 
 from sqlalchemy.orm.interfaces import MANYTOMANY
 from sqlalchemy.orm.interfaces import ONETOMANY
+from sqlalchemy.orm.interfaces import MANYTOONE
 from sqlalchemy.orm import class_mapper, Query
 from sqlalchemy.orm.attributes import ScalarAttributeImpl, ScalarObjectAttributeImpl, CollectionAttributeImpl
 from sqlalchemy.orm.properties import CompositeProperty, ColumnProperty
@@ -21,7 +22,7 @@ try:
     from sqlalchemy import exc as sqlalchemy_exceptions
 except ImportError:
     from sqlalchemy import exceptions as sqlalchemy_exceptions
-from sqlalchemy.orm import object_session
+from sqlalchemy.orm import compile_mappers, object_session, class_mapper
 from formalchemy import helpers as h
 from formalchemy import fatypes, validators
 from formalchemy.exceptions import FieldNotFoundError
@@ -1010,7 +1011,7 @@ class AbstractField(object):
     _valide_options = [
             'validate', 'renderer', 'hidden', 'required', 'readonly',
             'null_as', 'label', 'multiple', 'options', 'validators',
-            'size', 'instructions', 'metadata', 'html']
+            'size', 'instructions', 'metadata', 'html', 'attrs']
 
     def __init__(self, parent, name=None, type=fatypes.String, **kwattrs):
         # the FieldSet (or any ModelRenderer) owning this instance
@@ -1045,11 +1046,7 @@ class AbstractField(object):
         wrapper.render_opts = dict(self.render_opts)
         wrapper.validators = list(self.validators)
         wrapper.errors = list(self.errors)
-        try:
-            wrapper._renderer = copy(self._renderer)
-        except TypeError: # 2.4 support
-            # it's a lambda, safe to just use same reference
-            pass
+        wrapper._renderer = copy(self._renderer)
         if hasattr(wrapper._renderer, 'field'):
             wrapper._renderer.field = wrapper
         return wrapper
@@ -1087,7 +1084,7 @@ class AbstractField(object):
         if self.is_required() and validators.required not in L:
             L.append(validators.required)
         for validator in L:
-            if (not (hasattr(validator, 'accepts_none') and validator.accepts_none)) and value is None:
+            if value is None and not getattr(validator, 'accepts_none', False):
                 continue
             try:
                 validator(value, self)
@@ -1203,7 +1200,7 @@ class AbstractField(object):
                 else:
                     renderer = HiddenFieldRenderer
                 self._renderer = renderer
-            elif attr in 'attrs':
+            elif attr == 'attrs':
                 self.render_opts.update(value)
             elif attr in mapping:
                 attr = mapping.get(attr)
@@ -1516,7 +1513,7 @@ class AbstractField(object):
                 return self.parent.default_renderers[t]
         raise TypeError(
                 'No renderer found for field %s. '
-                'Type %s as no default renderer' % (self.name, self.type))
+                'Type %s has no default renderer' % (self.name, self.type))
 
     @property
     def renderer(self):
