@@ -6,13 +6,13 @@ __doc__ = r"""
 # some low-level testing first
 
 >>> fs = FieldSet(order1)
->>> fs._raw_fields()
+>>> list(fs._raw_fields())
 [AttributeField(id), AttributeField(user_id), AttributeField(quantity), AttributeField(user)]
 >>> fs.user.name
 'user_id'
 
 >>> fs = FieldSet(bill)
->>> fs._raw_fields()
+>>> list(fs._raw_fields())
 [AttributeField(id), AttributeField(email), AttributeField(password), AttributeField(name), AttributeField(orders)]
 >>> fs.orders.name
 'orders'
@@ -20,11 +20,11 @@ __doc__ = r"""
 binding should not change attribute order:
 >>> fs = FieldSet(User)
 >>> fs_bound = fs.bind(User)
->>> fs_bound._fields.values()
-[AttributeField(id), AttributeField(email), AttributeField(password), AttributeField(name), AttributeField(orders)]
+>>> sorted(fs_bound._fields.items())
+[('email', AttributeField(email)), ('id', AttributeField(id)), ('name', AttributeField(name)), ('orders', AttributeField(orders)), ('password', AttributeField(password))]
 
 >>> fs = FieldSet(User2)
->>> fs._raw_fields()
+>>> list(fs._raw_fields())
 [AttributeField(user_id), AttributeField(address_id), AttributeField(name), AttributeField(address)]
 
 >>> fs.render() #doctest: +ELLIPSIS
@@ -36,7 +36,7 @@ Exception: No session found...
 >>> fs.configure(pk=True, focus=None)
 >>> fs.id.is_required()
 True
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="One--id">
   Id
@@ -45,8 +45,12 @@ True
 </div>
 
 >>> fs = FieldSet(Two)
+>>> fs
+<FieldSet with ['id', 'foo']>
 >>> fs.configure(pk=True)
->>> print fs.render()
+>>> fs
+<FieldSet (configured) with ['id', 'foo']>
+>>> print(fs.render())
 <div>
  <label class="field_req" for="Two--id">
   Id
@@ -66,7 +70,7 @@ document.getElementById("Two--id").focus();
 </div>
 
 >>> fs = FieldSet(Two)
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_opt" for="Two--foo">
   Foo
@@ -81,7 +85,7 @@ document.getElementById("Two--foo").focus();
 
 >>> fs = FieldSet(Two)
 >>> fs.configure(options=[fs.foo.label('A custom label')])
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_opt" for="Two--foo">
   A custom label
@@ -94,7 +98,7 @@ document.getElementById("Two--foo").focus();
 //]]>
 </script>
 >>> fs.configure(options=[fs.foo.label('')])
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_opt" for="Two--foo">
  </label>
@@ -112,12 +116,12 @@ document.getElementById("Two--foo").focus();
 
 >>> fs = FieldSet(Two)
 >>> fs.configure(include=[fs.foo.hidden()])
->>> print fs.render()
+>>> print(fs.render())
 <input id="Two--foo" name="Two--foo" type="hidden" value="133" />
 
 >>> fs = FieldSet(Two)
 >>> fs.configure(include=[fs.foo.dropdown([('option1', 'value1'), ('option2', 'value2')])])
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_opt" for="Two--foo">
   Foo
@@ -139,7 +143,7 @@ document.getElementById("Two--foo").focus();
 
 >>> fs = FieldSet(Two)
 >>> assert configure_and_render(fs, include=[fs.foo.dropdown([('option1', 'value1'), ('option2', 'value2')])]) == configure_and_render(fs, options=[fs.foo.dropdown([('option1', 'value1'), ('option2', 'value2')])]) 
->>> print pretty_html(fs.foo.with_html(onblur='test()').render())
+>>> print(pretty_html(fs.foo.with_html(onblur='test()').render()))
 <select id="Two--foo" name="Two--foo" onblur="test()">
  <option value="value1">
   option1
@@ -148,13 +152,13 @@ document.getElementById("Two--foo").focus();
   option2
  </option>
 </select>
->>> print fs.foo.reset().with_html(onblur='test').render()
+>>> print(fs.foo.reset().with_html(onblur='test').render())
 <input id="Two--foo" name="Two--foo" onblur="test" type="text" value="133" />
 
 # Test with_metadata()
 >>> fs = FieldSet(Three)
 >>> fs.configure(include=[fs.foo.with_metadata(instructions=u'Answer well')])
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_opt" for="Three--foo">
   Foo
@@ -171,27 +175,42 @@ document.getElementById("Three--foo").focus();
 </script>
 
 # test sync
->>> print session.query(One).count()
+>>> print(session.query(One).count())
 0
->>> fs_1 = FieldSet(One, data={})
+>>> fs_1 = FieldSet(One, data={}, session=session)
 >>> fs_1.sync()
 >>> session.flush()
->>> print session.query(One).count()
+>>> print(session.query(One).count())
 1
 >>> session.rollback()
 
 >>> twof = TwoFloat(id=1, foo=32.2)
 >>> fs_twof = FieldSet(twof)
->>> fs_twof.foo.value
-32.200000000000003
->>> print pretty_html(fs_twof.foo.render())
+>>> print('%.1f' % fs_twof.foo.value)
+32.2
+>>> print(pretty_html(fs_twof.foo.render()))
 <input id="TwoFloat-1-foo" name="TwoFloat-1-foo" type="text" value="32.2" />
 
+>>> import datetime
+>>> twoi = TwoInterval(id=1, foo=datetime.timedelta(2.2))
+>>> fs_twoi = FieldSet(twoi)
+>>> fs_twoi.foo.renderer
+<IntervalFieldRenderer for AttributeField(foo)>
+>>> fs_twoi.foo.value
+datetime.timedelta(2, 17280)
+>>> print(pretty_html(fs_twoi.foo.render()))
+<input id="TwoInterval-1-foo" name="TwoInterval-1-foo" type="text" value="2.17280" />
+>>> fs_twoi.rebind(data={"TwoInterval-1-foo": "3.1"})
+>>> fs_twoi.sync()
+>>> new_twoi = fs_twoi.model
+>>> new_twoi.foo == datetime.timedelta(3.1)
+True
+
 # test render and sync fatypes.Numeric
 # http://code.google.com/p/formalchemy/issues/detail?id=41
 >>> twon = TwoNumeric(id=1, foo=Decimal('2.3'))
 >>> fs_twon = FieldSet(twon)
->>> print pretty_html(fs_twon.foo.render())
+>>> print(pretty_html(fs_twon.foo.render()))
 <input id="TwoNumeric-1-foo" name="TwoNumeric-1-foo" type="text" value="2.3" />
 >>> fs_twon.rebind(data={"TwoNumeric-1-foo": "6.7"})
 >>> fs_twon.sync()
@@ -209,7 +228,7 @@ True
 >>> fs_cb = FieldSet(CheckBox)
 >>> fs_cb.field.value is None
 True
->>> print pretty_html(fs_cb.field.dropdown().render())
+>>> print(pretty_html(fs_cb.field.dropdown().render()))
 <select id="CheckBox--field" name="CheckBox--field">
  <option value="True">
   Yes
@@ -225,15 +244,15 @@ True
 True
 >>> fs_cb.field.value
 False
->>> fs_cb.field.renderer._value is None
+>>> fs_cb.field.renderer.value is None
 True
->>> print fs_cb.field.render()
+>>> print(fs_cb.field.render())
 <input id="CheckBox--field" name="CheckBox--field" type="checkbox" value="True" />
 >>> fs_cb.field.renderer #doctest: +ELLIPSIS
 <CheckBoxFieldRenderer for AttributeField(field)>
 >>> fs_cb.field.renderer._serialized_value() is None
 True
->>> print pretty_html(fs_cb.field.radio().render())
+>>> print(pretty_html(fs_cb.field.radio().render()))
 <input id="CheckBox--field_0" name="CheckBox--field" type="radio" value="True" />
 <label for="CheckBox--field_0">
  Yes
@@ -265,7 +284,7 @@ True
 False
 
 >>> fs = FieldSet(Two)
->>> print pretty_html(fs.foo.dropdown(options=['one', 'two']).radio().render())
+>>> print(pretty_html(fs.foo.dropdown(options=['one', 'two']).radio().render()))
 <input id="Two--foo_0" name="Two--foo" type="radio" value="one" />
 <label for="Two--foo_0">
  one
@@ -277,11 +296,14 @@ False
 </label>
 
 >>> assert fs.foo.radio(options=['one', 'two']).render() == fs.foo.dropdown(options=['one', 'two']).radio().render()
->>> print fs.foo.radio(options=['one', 'two']).dropdown().render()
-<select id="Two--foo" name="Two--foo"><option value="one">one</option>
-<option value="two">two</option></select>
->>> assert fs.foo.dropdown(options=['one', 'two']).render() == fs.foo.radio(options=['one', 'two']).dropdown().render()
->>> print pretty_html(fs.foo.dropdown(options=['one', 'two'], multiple=True).checkbox().render())
+>>> print(fs.foo.radio(options=[('one','one'), ('two','two')]).dropdown().render())
+<select id="Two--foo" name="Two--foo">
+<option value="one">one</option>
+<option value="two">two</option>
+</select>
+
+>>> assert fs.foo.dropdown(options=[('one','one'), ('two','two')]).render() == fs.foo.radio(options=[('one','one'), ('two','two')]).dropdown().render()
+>>> print(pretty_html(fs.foo.dropdown(options=[('one','one'), ('two','two')], multiple=True).checkbox().render()))
 <input id="Two--foo_0" name="Two--foo" type="checkbox" value="one" />
 <label for="Two--foo_0">
  one
@@ -292,8 +314,8 @@ False
  two
 </label>
 
->>> fs = FieldSet(User)
->>> print fs.render()
+>>> fs = FieldSet(User, session=session)
+>>> print(fs.render())
 <div>
  <label class="field_req" for="User--email">
   Email
@@ -333,11 +355,9 @@ document.getElementById("User--email").focus();
   </option>
  </select>
 </div>
->>> FieldSet(User).render() == FieldSet(User, session).render()
-True
 
 >>> fs = FieldSet(bill)
->>> print pretty_html(fs.orders.render())
+>>> print(pretty_html(fs.orders.render()))
 <select id="User-1-orders" multiple="multiple" name="User-1-orders" size="5">
  <option value="2">
   Quantity: 5
@@ -345,11 +365,11 @@ True
  <option value="3">
   Quantity: 6
  </option>
- <option value="1" selected="selected">
+ <option selected="selected" value="1">
   Quantity: 10
  </option>
 </select>
->>> print pretty_html(fs.orders.checkbox().render())
+>>> print(pretty_html(fs.orders.checkbox().render()))
 <input id="User-1-orders_0" name="User-1-orders" type="checkbox" value="2" />
 <label for="User-1-orders_0">
  Quantity: 5
@@ -365,7 +385,7 @@ True
  Quantity: 10
 </label>
 
->>> print fs.orders.checkbox(options=session.query(Order).filter_by(id=1)).render()
+>>> print(fs.orders.checkbox(options=session.query(Order).filter_by(id=1)).render())
 <input checked="checked" id="User-1-orders_0" name="User-1-orders" type="checkbox" value="1" /><label for="User-1-orders_0">Quantity: 10</label>
 
 >>> fs = FieldSet(bill, data={})
@@ -374,12 +394,12 @@ True
 True
 
 >>> fs = FieldSet(bill, data={'User-1-orders': ['2', '3']})
->>> print pretty_html(fs.orders.render())
+>>> print(pretty_html(fs.orders.render()))
 <select id="User-1-orders" multiple="multiple" name="User-1-orders" size="5">
- <option value="2" selected="selected">
+ <option selected="selected" value="2">
   Quantity: 5
  </option>
- <option value="3" selected="selected">
+ <option selected="selected" value="3">
   Quantity: 6
  </option>
  <option value="1">
@@ -387,17 +407,24 @@ True
  </option>
 </select>
 
+>>> fs.orders.model_value
+[1]
+>>> fs.orders.raw_value
+[<Order for user 1: 10>]
+
 >>> fs = FieldSet(Two)
->>> print fs.foo.render()
+>>> print(fs.foo.render())
 <input id="Two--foo" name="Two--foo" type="text" value="133" />
 
 >>> fs = FieldSet(Two)
->>> print fs.foo.dropdown([('option1', 'value1'), ('option2', 'value2')]).render()
-<select id="Two--foo" name="Two--foo"><option value="value1">option1</option>
-<option value="value2">option2</option></select>
+>>> print(fs.foo.dropdown([('option1', 'value1'), ('option2', 'value2')]).render())
+<select id="Two--foo" name="Two--foo">
+<option value="value1">option1</option>
+<option value="value2">option2</option>
+</select>
 
 >>> fs = FieldSet(Order, session)
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="Order--quantity">
   Quantity
@@ -436,7 +463,7 @@ document.getElementById("Order--quantity").focus();
 10
 >>> fs.session == object_session(order1)
 True
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="Order-1-id">
   Id
@@ -454,7 +481,7 @@ document.getElementById("Order-1-id").focus();
   User
  </label>
  <select id="Order-1-user_id" name="Order-1-user_id">
-  <option value="1" selected="selected">
+  <option selected="selected" value="1">
    Bill
   </option>
   <option value="2">
@@ -465,7 +492,7 @@ document.getElementById("Order-1-id").focus();
 
 >>> fs = FieldSet(One)
 >>> fs.configure(pk=True)
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="One--id">
   Id
@@ -478,10 +505,10 @@ document.getElementById("One--id").focus();
 //]]>
 </script>
 >>> fs.configure(include=[])
->>> print fs.render()
+>>> print(fs.render())
 <BLANKLINE>
 >>> fs.configure(pk=True, focus=None)
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="One--id">
   Id
@@ -498,16 +525,19 @@ ValueError: ...
 >>> fs = FieldSet(Two)
 >>> fs.configure()
 >>> fs2 = fs.bind(Two)
->>> [fs2 == field.parent for field in fs2._render_fields.itervalues()]
+>>> [fs2 == field.parent for field in fs2._render_fields.values()]
 [True]
 
 >>> fs = FieldSet(OTOParent, session)
->>> print fs.render()
+>>> print(fs.render())
 <div>
  <label class="field_req" for="OTOParent--oto_child_id">
   Child
  </label>
  <select id="OTOParent--oto_child_id" name="OTOParent--oto_child_id">
+  <option value="1">
+   baz
+  </option>
  </select>
 </div>
 <script type="text/javascript">
@@ -516,8 +546,12 @@ document.getElementById("OTOParent--oto_child_id").focus();
 //]]>
 </script>
 
+>>> fs.rebind(parent)
+>>> fs.child.raw_value
+<OTOChild baz>
+
 # validation + sync
->>> fs_2 = FieldSet(Two, data={'Two--foo': ''})
+>>> fs_2 = FieldSet(Two, session=session, data={'Two--foo': ''})
 >>> fs_2.foo.value # '' is deserialized to None, so default of 133 is used
 '133'
 >>> fs_2.validate()
@@ -527,7 +561,7 @@ True
 False
 >>> fs_2.errors
 {AttributeField(foo): ['Please enter a value']}
->>> print fs_2.render()
+>>> print(fs_2.render())
 <div>
  <label class="field_req" for="Two--foo">
   Foo
@@ -539,12 +573,12 @@ False
 </div>
 >>> fs_2.rebind(data={'Two--foo': 'asdf'})
 >>> fs_2.data
-{'Two--foo': 'asdf'}
+SimpleMultiDict([('Two--foo', 'asdf')])
 >>> fs_2.validate()
 False
 >>> fs_2.errors
-{AttributeField(foo): [ValidationError('Value is not an integer',)]}
->>> print fs_2.render()
+{AttributeField(foo): ['Value is not an integer']}
+>>> print(fs_2.render())
 <div>
  <label class="field_req" for="Two--foo">
   Foo
@@ -556,7 +590,7 @@ False
 </div>
 >>> fs_2.rebind(data={'Two--foo': '2'})
 >>> fs_2.data
-{'Two--foo': '2'}
+SimpleMultiDict([('Two--foo', '2')])
 >>> fs_2.validate()
 True
 >>> fs_2.errors
@@ -565,13 +599,13 @@ True
 >>> fs_2.model.foo
 2
 >>> session.flush()
->>> print fs_2.render() #doctest: +ELLIPSIS
+>>> print(fs_2.render()) #doctest: +ELLIPSIS,+IGNORE_EXCEPTION_DETAIL
 Traceback (most recent call last):
 ...
-Exception: Primary key of model has changed since binding, probably due to sync()ing a new instance.  You can solve this by either binding to a model with the original primary key again, or by binding data to None.
+PkError: Primary key of model has changed since binding, probably due to sync()ing a new instance (from None to 1)...
 >>> session.rollback()
 
->>> fs_1 = FieldSet(One, data={'One--id': '1'})
+>>> fs_1 = FieldSet(One, session=session, data={'One--id': '1'})
 >>> fs_1.configure(pk=True)
 >>> fs_1.validate()
 True
@@ -579,24 +613,24 @@ True
 >>> fs_1.model.id
 1
 >>> fs_1.rebind(data={'One--id': 'asdf'})
->>> fs_1.id.renderer.name
-u'One--id'
+>>> str(fs_1.id.renderer.name)
+'One--id'
 >>> fs_1.validate()
 False
 >>> fs_1.errors
-{AttributeField(id): [ValidationError('Value is not an integer',)]}
+{AttributeField(id): ['Value is not an integer']}
 
 # test updating _bound_pk copy
 >>> one = One(id=1)
 >>> fs_11 = FieldSet(one)
->>> fs_11.id.renderer.name
-u'One-1-id'
+>>> str(fs_11.id.renderer.name)
+'One-1-id'
 >>> one.id = 2
 >>> fs_11.rebind(one)
->>> fs_11.id.renderer.name
-u'One-2-id'
+>>> str(fs_11.id.renderer.name)
+'One-2-id'
 
->>> fs_u = FieldSet(User, data={})
+>>> fs_u = FieldSet(User, session=session, data={})
 >>> fs_u.configure(include=[fs_u.orders])
 >>> fs_u.validate()
 True
@@ -612,16 +646,16 @@ True
 >>> session.rollback()
 
 >>> fs_3 = FieldSet(Three, data={'Three--foo': 'asdf', 'Three--bar': 'fdsa'})
->>> fs_3.foo.value
-'asdf'
->>> print fs_3.foo.textarea().render()
+>>> print(fs_3.foo.value)
+asdf
+>>> print(fs_3.foo.textarea().render())
 <textarea id="Three--foo" name="Three--foo">asdf</textarea>
->>> print fs_3.foo.textarea("3x4").render()
+>>> print(fs_3.foo.textarea("3x4").render())
 <textarea cols="3" id="Three--foo" name="Three--foo" rows="4">asdf</textarea>
->>> print fs_3.foo.textarea((3,4)).render()
+>>> print(fs_3.foo.textarea((3,4)).render())
 <textarea cols="3" id="Three--foo" name="Three--foo" rows="4">asdf</textarea>
->>> fs_3.bar.value
-'fdsa'
+>>> print(fs_3.bar.value)
+fdsa
 >>> def custom_validator(fs):
 ...   if fs.foo.value != fs.bar.value:
 ...     fs.foo.errors.append('does not match bar')
@@ -629,9 +663,9 @@ True
 >>> fs_3.configure(global_validator=custom_validator, focus=None)
 >>> fs_3.validate()
 False
->>> sorted(fs_3.errors.items())
-[(None, ('foo and bar do not match',)), (AttributeField(foo), ['does not match bar'])]
->>> print fs_3.render()
+>>> sorted((k.key if k is not None else 'NONE', v) for k,v in fs_3.errors.items())
+[('NONE', ('foo and bar do not match',)), ('foo', ['does not match bar'])]
+>>> print(fs_3.render())
 <div class="fieldset_error">
  foo and bar do not match
 </div>
@@ -654,12 +688,12 @@ False
 # custom renderer
 >>> fs_3 = FieldSet(Three, data={'Three--foo': 'http://example.com/image.png'})
 >>> fs_3.configure(include=[fs_3.foo.with_renderer(ImgRenderer)])
->>> print fs_3.foo.render()
+>>> print(fs_3.foo.render())
 <img src="http://example.com/image.png">
 
 # natural PKs
 >>> fs_npk = FieldSet(NaturalOrder, session)
->>> print fs_npk.render()
+>>> print(fs_npk.render())
 <div>
  <label class="field_req" for="NaturalOrder--quantity">
   Quantity
@@ -685,8 +719,8 @@ document.getElementById("NaturalOrder--quantity").focus();
  </select>
 </div>
 >>> fs_npk.rebind(norder2, session, data={'NaturalOrder-2-user_email': nbill.email, 'NaturalOrder-2-quantity': str(norder2.quantity)})
->>> fs_npk.user_email.renderer.name
-u'NaturalOrder-2-user_email'
+>>> str(fs_npk.user_email.renderer.name)
+'NaturalOrder-2-user_email'
 >>> fs_npk.sync()
 >>> fs_npk.model.user_email == nbill.email
 True
@@ -706,7 +740,7 @@ True
 # Field
 >>> fs = FieldSet(One)
 >>> fs.add(Field('foo'))
->>> print configure_and_render(fs, focus=None)
+>>> print(configure_and_render(fs, focus=None))
 <div>
  <label class="field_opt" for="One--foo">
   Foo
@@ -718,7 +752,7 @@ True
 >>> fs.add(Field('foo', types.Integer, value=2))
 >>> fs.foo.value
 2
->>> print configure_and_render(fs, focus=None)
+>>> print(configure_and_render(fs, focus=None))
 <div>
  <label class="field_opt" for="One--foo">
   Foo
@@ -730,41 +764,9 @@ True
 >>> fs.foo.value
 4
 
->>> fs = FieldSet(Manual)
->>> print configure_and_render(fs, focus=None)
-<div>
- <label class="field_opt" for="Manual--a">
-  A
- </label>
- <input id="Manual--a" name="Manual--a" type="text" />
-</div>
-<div>
- <label class="field_opt" for="Manual--b">
-  B
- </label>
- <select id="Manual--b" multiple="multiple" name="Manual--b" size="5">
-  <option value="1">
-   one
-  </option>
-  <option value="2">
-   two
-  </option>
- </select>
-</div>
-<div>
- <label class="field_opt" for="Manual--d">
-  D
- </label>
- <textarea cols="80" id="Manual--d" name="Manual--d" rows="10">
- </textarea>
-</div>
->>> fs.rebind(data={'Manual--a': 'asdf'})
->>> print pretty_html(fs.a.render())
-<input id="Manual--a" name="Manual--a" type="text" value="asdf" />
-
 >>> fs = FieldSet(One)
 >>> fs.add(Field('foo', types.Integer, value=2).dropdown(options=[('1', 1), ('2', 2)]))
->>> print configure_and_render(fs, focus=None)
+>>> print(configure_and_render(fs, focus=None))
 <div>
  <label class="field_opt" for="One--foo">
   Foo
@@ -773,7 +775,7 @@ True
   <option value="1">
    1
   </option>
-  <option value="2" selected="selected">
+  <option selected="selected" value="2">
    2
   </option>
  </select>
@@ -791,7 +793,7 @@ True
 
 >>> fs_1 = FieldSet(One)
 >>> fs_1.add(Field('foo', types.Integer, value=[2, 3]).dropdown(options=[('1', 1), ('2', 2), ('3', 3)], multiple=True))
->>> print configure_and_render(fs_1, focus=None)
+>>> print(configure_and_render(fs_1, focus=None))
 <div>
  <label class="field_opt" for="One--foo">
   Foo
@@ -800,10 +802,10 @@ True
   <option value="1">
    1
   </option>
-  <option value="2" selected="selected">
+  <option selected="selected" value="2">
    2
   </option>
-  <option value="3" selected="selected">
+  <option selected="selected" value="3">
    3
   </option>
  </select>
@@ -831,9 +833,11 @@ False
 >>> d[types.Boolean] = BooleanSelectRenderer
 >>> fs = FieldSet(CheckBox)
 >>> fs.default_renderers = d
->>> print fs.field.render()
-<select id="CheckBox--field" name="CheckBox--field"><option value="True">Yes</option>
-<option value="False">No</option></select>
+>>> print(fs.field.render())
+<select id="CheckBox--field" name="CheckBox--field">
+<option value="True">Yes</option>
+<option value="False">No</option>
+</select>
 
 # test setter rejection
 >>> fs = FieldSet(One)
@@ -844,10 +848,10 @@ AttributeError: Do not set field attributes manually.  Use append() or configure
 
 # join
 >>> fs = FieldSet(Order__User)
->>> fs._fields.values()
-[AttributeField(orders_id), AttributeField(orders_user_id), AttributeField(orders_quantity), AttributeField(users_id), AttributeField(users_email), AttributeField(users_password), AttributeField(users_name)]
+>>> sorted((str(k),v) for k,v in fs._fields.items())
+[('orders_id', AttributeField(orders_id)), ('orders_quantity', AttributeField(orders_quantity)), ('orders_user_id', AttributeField(orders_user_id)), ('users_email', AttributeField(users_email)), ('users_id', AttributeField(users_id)), ('users_name', AttributeField(users_name)), ('users_password', AttributeField(users_password))]
 >>> fs.rebind(session.query(Order__User).filter_by(orders_id=1).one())
->>> print configure_and_render(fs, focus=None)
+>>> print(configure_and_render(fs, focus=None))
 <div>
  <label class="field_req" for="Order__User-1_1-orders_quantity">
   Orders quantity
@@ -884,23 +888,22 @@ True
 
 >>> FieldSet.default_renderers[Point] = PointFieldRenderer
 >>> fs = FieldSet(Vertex)
->>> print pretty_html(fs.start.render())
+>>> print(pretty_html(fs.start.render()))
 <input id="Vertex--start-x" name="Vertex--start-x" type="text" value="" />
 <input id="Vertex--start-y" name="Vertex--start-y" type="text" value="" />
 >>> fs.rebind(Vertex)
 >>> v = fs.model
 >>> v.start = Point(1,2)
 >>> v.end = Point(3,4)
->>> print pretty_html(fs.start.render())
+>>> print(pretty_html(fs.start.render()))
 <input id="Vertex--start-x" name="Vertex--start-x" type="text" value="1" />
 <input id="Vertex--start-y" name="Vertex--start-y" type="text" value="2" />
->>> fs.rebind(v) # this exercises a session bugfix
->>> fs.session == session
-True
+>>> fs.rebind(v)
 >>> fs.rebind(data={'Vertex--start-x': '10', 'Vertex--start-y': '20', 'Vertex--end-x': '30', 'Vertex--end-y': '40'})
 >>> fs.validate()
 True
 >>> fs.sync()
+>>> session.add(v)
 >>> session.flush()
 >>> v.id
 1
@@ -917,7 +920,7 @@ True
 >>> t.configure(readonly=True)
 >>> t.readonly
 True
->>> print t.render()
+>>> print(t.render())
 <tbody>
  <tr>
   <td class="field_readonly">
@@ -955,50 +958,21 @@ True
 >>> session.refresh(john)
 
 >>> fs_or = FieldSet(order1)
->>> print fs_or.user.render_readonly()
-Bill
+>>> print(fs_or.user.render_readonly())
+<a href="mailto:bill@example.com">Bill</a>
 
->>> t = FieldSet(Manual)
->>> t.configure(include=[t.a, t.b], readonly=True)
->>> t.model.b = [1, 2]
->>> print t.render()
-<tbody>
- <tr>
-  <td class="field_readonly">
-   A:
-  </td>
-  <td>
-  </td>
- </tr>
- <tr>
-  <td class="field_readonly">
-   B:
-  </td>
-  <td>
-   one, two
-  </td>
- </tr>
-</tbody>
->>> t.model.a = 'test'
->>> print t.a.render_readonly()
-test
->>> t.configure(readonly=True, options=[t.a.with_renderer(EscapingReadonlyRenderer)])
->>> t.model.a = '<test>'
->>> print t.a.render_readonly()
-&lt;test&gt;
-
->>> out = FieldSet(OrderUserTag)
->>> list(sorted(out._fields))
+>>> out = FieldSet(OrderUserTag, session=session)
+>>> list(str(x) for x in sorted(out._fields))
 ['id', 'order_id', 'order_user', 'tag', 'user_id']
->>> print out.order_user.name
+>>> print(out.order_user.name)
 order_user
 >>> out.order_user.is_raw_foreign_key
 False
 >>> out.order_user.is_composite_foreign_key
 True
->>> list(sorted(out.render_fields))
+>>> list(str(x) for x in sorted(out.render_fields))
 ['order_user', 'tag']
->>> print pretty_html(out.order_user.render())
+>>> print(pretty_html(out.order_user.render()))
 <select id="OrderUserTag--order_user" name="OrderUserTag--order_user">
  <option value="(1, 1)">
   OrderUser(1, 1)
@@ -1011,7 +985,7 @@ True
 >>> out.validate()
 True
 >>> out.sync()
->>> print out.model.order_user
+>>> print(out.model.order_user)
 OrderUser(1, 2)
 
 >>> fs = FieldSet(Function)
@@ -1026,13 +1000,13 @@ True
 >>> fs_r.validate()
 True
 
->>> fs_oo = FieldSet(OptionalOrder)
+>>> fs_oo = FieldSet(OptionalOrder, session=session)
 >>> fs_oo.configure(options=[fs_oo.user.with_null_as(('No user', ''))])
 >>> fs_oo.user._null_option
 ('No user', '')
->>> print pretty_html(fs_oo.user.render())
+>>> print(pretty_html(fs_oo.user.render()))
 <select id="OptionalOrder--user_id" name="OptionalOrder--user_id">
- <option value="">
+ <option selected="selected" value="">
   No user
  </option>
  <option value="1">
@@ -1054,11 +1028,19 @@ True
 >>> fs_bad.configure(include=[Field('invalid')])
 Traceback (most recent call last):
 ...
-ValueError: Unrecognized Field `AttributeField(invalid)` in `include` -- did you mean to call append() first?
+ValueError: Unrecognized Field `Field(invalid)` in `include` -- did you mean to call append() first?
 
 >>> fs_s = FieldSet(Synonym)
->>> fs_s._fields
-{'foo': AttributeField(foo), 'id': AttributeField(id)}
+>>> sorted(fs_s._fields.items())
+[('_foo', AttributeField(_foo)), ('id', AttributeField(id))]
+
+>>> fs_s = FieldSet(AliasedRecursive)
+>>> sorted(fs_s._fields.items())
+[('foo', AttributeField(foo)), ('id', AttributeField(id)), ('parent', AttributeField(parent)), ('parent_id', AttributeField(parent_id))]
+
+>>> fs_s = FieldSet(RecursiveChild)
+>>> sorted(fs_s._fields.items())
+[('bar', AttributeField(bar)), ('foo', AttributeField(foo)), ('id', AttributeField(id)), ('parent', AttributeField(parent)), ('parent_id', AttributeField(parent_id)), ('typ_id', AttributeField(typ_id))]
 
 >>> fs_prefix = FieldSet(Two, prefix="myprefix")
 >>> print(fs_prefix.render())
@@ -1083,7 +1065,7 @@ True
 
 >>> fs_two = FieldSet(Two)
 >>> fs_two.configure(options=[fs_two.foo.label('1 < 2')])
->>> print fs_two.render()
+>>> print(fs_two.render())
 <div>
  <label class="field_opt" for="Two--foo">
   1 &lt; 2
@@ -1102,10 +1084,60 @@ True
 
 >>> fs_conflict = FieldSet(ConflictNames)
 >>> fs_conflict.rebind(conflict_names)
->>> print fs_conflict.render() #doctest: +ELLIPSIS
+>>> print(fs_conflict.render()) #doctest: +ELLIPSIS
 <div>
 ...
 
+>>> fs_syn = FieldSet(Synonym)
+>>> print(fs_syn.render())
+<div>
+ <label class="field_opt" for="Synonym--_foo">
+  foo
+ </label>
+ <input id="Synonym--_foo" name="Synonym--_foo" type="text" />
+</div>
+<script type="text/javascript">
+ //<![CDATA[
+document.getElementById("Synonym--_foo").focus();
+//]]>
+</script>
+
+>>> fs_rec = FieldSet(RecursiveChild, session=session)
+>>> print(fs_rec.render()) #doctest: +ELLIPSIS
+<div>
+ <label class="field_req" for="RecursiveChild--typ_id">
+  Typ id
+ </label>
+ <input id="RecursiveChild--typ_id" name="RecursiveChild--typ_id" type="text" value="42" />
+</div>
+<script type="text/javascript">
+ //<![CDATA[
+document.getElementById("RecursiveChild--typ_id").focus();
+//]]>
+</script>
+<div>
+ <label class="field_opt" for="RecursiveChild--foo">
+  Foo
+ </label>
+ <input id="RecursiveChild--foo" name="RecursiveChild--foo" type="text" />
+</div>
+<div>
+ <label class="field_opt" for="RecursiveChild--bar">
+  Bar
+ </label>
+ <input id="RecursiveChild--bar" name="RecursiveChild--bar" type="text" />
+</div>
+<div>
+ <label class="field_opt" for="RecursiveChild--parent_id">
+  Parent
+ </label>
+ <select id="RecursiveChild--parent_id" name="RecursiveChild--parent_id">
+  <option selected="selected" value="">
+   None
+  </option>
+ </select>
+</div>
+
 """
 
 if __name__ == '__main__':
